<?php
namespace Ep\TecDoc;

use Ep\TecDoc\Exception\HttpException;
use Exception;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use function GuzzleHttp\Psr7\str;
use Psr\Cache\CacheItemPoolInterface;

/**
 * TecDoc Servis
 *
 * Kroz ovu klasu se dohvaćaju svi podaci iz TecDoc-a.
 *
 * @package Ep\TecDoc
 */
class TecDocService
{
	const SERVICE_URI = 'https://webservice.tecalliance.services';
	const SERVICE_ENDPOINT = '/pegasus-3-0/services/TecdocToCatDLB.jsonEndpoint';

	/** @var array */
	protected $config;

	/** @var ClientInterface */
	protected $client;

	/** @var CacheItemPoolInterface */
	protected $cache;

	/** @var array */
	protected $selectedVehicle;

	/**
	 * Konstruktor.
	 *
	 * @param array $config
	 * @param ClientInterface $client
	 * @param CacheItemPoolInterface $cache
	 */
    public function __construct(array $config = [], ClientInterface $client, CacheItemPoolInterface $cache)
    {
	    $this->config = $config;
	    $this->client = $client;
	    $this->cache = $cache;

	    $this->allowClientAccess();
    }

	/**
	 * Daje pristup klijentovom IP-u TecDocu-.
	 */
    protected function allowClientAccess()
    {
    	if (!isset($_SESSION['tecdoc_allow']) || (time() - $_SESSION['tecdoc_allow']) > 23 * 3600 || $_SESSION['tecdoc_allow_ip'] != $_SERVER['REMOTE_ADDR']) {
    		if (!in_array($_SERVER['REMOTE_ADDR'], ['127.0.0.1', '::1'])) {
    			if (!isset($_SESSION['tecdoc_allow_ip']) || $_SESSION['tecdoc_allow_ip'] != $_SERVER['REMOTE_ADDR']) {
				    $this->request('addDynamicAddress', [
					    'address'       => $_SERVER['REMOTE_ADDR'],
					    'validityHours' => 24,
				    ]);

				    $_SESSION['tecdoc_allow'] = time();
				    $_SESSION['tecdoc_allow_ip'] = $_SERVER['REMOTE_ADDR'];
			    }
		    }
	    }
    }

	/**
     * Upućuje request prema TecDoc servisu.
     *
	 * @param string $method TecDoc metoda.
	 * @param array $params Parametri requesta.
	 *
	 * @return array|mixed
	 * @throws Exception
	 */
    public function request($method, array $params = [])
    {
	    $params = [$method => $this->checkParams($params)];

        try {
            $response = $this->client->request('POST', self::SERVICE_ENDPOINT, [
                'json' => $params,
		    ]);
	    } catch (GuzzleException $e) {
		    throw new HttpException(sprintf('TecDoc HTTP Greška: %s', $e->getMessage()), 0, $e);
	    }

	    if ($response->getStatusCode() > 200) {
		    throw new HttpException(sprintf(
			    'TecDoc HTTP Greška: %d, %s',
			    $response->getStatusCode(),
			    $response->getReasonPhrase()
		    ));
	    }

	    $jsonContent = (string) $response->getBody();
	    $responseData = json_decode($jsonContent, true);

        if (null === $responseData) {
        	throw new \RuntimeException(sprintf(
        		"Došlo je do pogreške prilikom dekodiranja JSON-a: %s.\nJSON: %s",
		        json_last_error_msg(),
		        $jsonContent
	        ));
        }

	    if ($responseData['status'] > 200) {
		    throw new HttpException(sprintf(
			    'TecDoc HTTP Greška: %d, %s',
			    $responseData['status'],
			    $responseData['statusText']
		    ));
	    }

	    return isset($responseData['data']['array']) && is_array($responseData['data']['array'])
		    ? $responseData['data']['array']
		    : [];
    }

    /**
     * Dodaje zajedničke parametre iz konfiguracije u request.
     *
     * @param array $params Parametri requesta.
     *
     * @return array
     */
    protected function checkParams(array $params)
    {
    	if (!isset($params['linkingTargetType'])) {
    		$params['linkingTargetType'] = $this->config['linkingTargetType'];
	    }

	    if (!isset($params['carType'])) {
    		$params['carType'] = $this->config['linkingTargetType'];
	    }

	    if (!isset($params['lang'])) {
    		$params['lang'] = $this->config['language'];
	    }

	    if (!isset($params['country'])) {
    		$params['country'] = $this->config['country'];
	    }

	    if (!isset($params['countriesCarSelection'])) {
    		$params['countriesCarSelection'] = $this->config['country'];
	    }

	    if (!isset($params['articleCountry'])) {
    		$params['articleCountry'] = $this->config['country'];
	    }

	    if (!isset($params['provider'])) {
    		$params['provider'] = $this->config['provider'];
	    }

	    return $params;
    }

	/**
	 * Vraća trenutno odabrano vozilo.
	 *
	 * @return mixed[]|null
	 */
    public function getSelectedVehicle()
    {
    	if (null === $this->selectedVehicle && isset($_COOKIE['_tecdoc'])) {
		    $this->selectedVehicle = $this->getVehicle($_COOKIE['_tecdoc']);
	    }

	    return $this->selectedVehicle;
    }

	/**
	 * Postavlja trenutno odabrano vozilo.
	 *
	 * @param string|array $vehicle
	 */
    public function setSelectedVehicle($vehicle)
    {
    	if (!is_array($vehicle)) {
    		$vehicle = $this->getVehicle($vehicle);
	    }

	    $current = $this->getSelectedVehicle();

    	if (!$current || $current['id'] != $vehicle['id']) {
    		setcookie('_tecdoc', $vehicle['slug'], time() + 365 * 86400, '/');
    		$this->selectedVehicle = $vehicle;
	    }
    }

    //<editor-fold desc="Proizvođači vozila">

    /**
     * Vraća proizvođače vozila.
     *
     * @return array[]
     */
    public function getManufacturers()
    {
    	$item = $this->cache->getItem('manufacturers.all');

    	if (!$item->isHit()) {
		    $manufacturers = $this->request('getManufacturers');
		    $config = $this->config;

		    $manufacturers = array_filter($manufacturers, function($manufacturer) use ($config) {
			    return !in_array($manufacturer['manuId'], $config['excludedManufacturerIds']);
		    });

		    $manufacturers = array_map(function($manufacturer) {
		    	if ('SKODA' === $manufacturer['manuName']) {
		    		$manufacturer['manuName'] = 'ŠKODA';
			    }

		    	return [
				    'id' => $manufacturer['manuId'],
				    'name' => $manufacturer['manuName'],
				    'slug' => sanitize_title_with_dashes(remove_accents($manufacturer['manuName'])),
			    ];
		    }, $manufacturers);

		    $item->set($manufacturers);
		    $this->cache->save($item);
	    }

	    return $item->get();
    }

	/**
	 * Vraća izdvojene proizvođače vozila.
	 *
	 * @return array[]
	 */
    public function getFeaturedManufacturers()
    {
	    $item = $this->cache->getItem('manufacturers.featured');

	    if (!$item->isHit()) {
	    	$config = $this->config;
	    	$featured = [];

	    	foreach ($config['featuredManufacturerIds'] as $id) {
	    	    $featured[] = $this->getManufacturer($id);
            }

	    	$item->set($featured);
	    	$this->cache->save($item);
	    }

	    return $item->get();
    }

	/**
	 * Vraća proizvođača po id-u ili slug-u.
	 *
	 * @param int|string $idOrSlug
	 * @return mixed
	 */
	public function getManufacturer($idOrSlug)
	{
		$field = is_numeric($idOrSlug) ? 'id' : 'slug';
		$cacheKey = 'id' === $field
			? sprintf('manufacturers.%s.%d', $field, intval($idOrSlug))
			: sprintf('manufacturers.%s.%s', $field, md5($idOrSlug));
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			foreach ($this->getManufacturers() as $manufacturer) {
				if ($manufacturer[$field] == $idOrSlug) {
					$item->set($manufacturer);
					$this->cache->save($item);
					break;
				}
			}
		}

		return $item->get();
	}

	//</editor-fold>
	//<editor-fold desc="Modeli vozila">

    /**
     * Vraća modele vozila za datog proizvođača.
     *
     * @param int|array $manufacturer
     *
     * @return array[]
     */
    public function getModels($manufacturer)
    {
    	if (!is_array($manufacturer)) {
		    $manufacturer = $this->getManufacturer($manufacturer);
	    }

    	$cacheKey = sprintf('models.%d', $manufacturer['id']);
    	$item = $this->cache->getItem($cacheKey);

    	if (!$item->isHit()) {
		    $raw = $this->request('getModelSeries', [
			    'manuId' => $manufacturer['id'],
		    ]);

		    $models = [];

		    foreach ($raw as $model) {
		    	$yearFrom = isset($model['yearOfConstrFrom'])
			        ? $this->formatConstructionYear($model['yearOfConstrFrom'])
			        : '...';

		    	$yearTo = isset($model['yearOfConstrTo'])
			        ? $this->formatConstructionYear($model['yearOfConstrTo'])
			        : '...';

		    	$year = sprintf('%s - %s', $yearFrom, $yearTo);
		    	$modelSlug = sprintf('%s-%d', sanitize_title_with_dashes(remove_accents($model['modelname'])), $model['modelId']);
		    	$slug = sprintf('%s--%s', $manufacturer['slug'], $modelSlug);

		    	$models[] = [
				    'id' => $model['modelId'],
				    'manufacturerId' => $manufacturer,
				    'name' => $model['modelname'],
				    'slug' => $slug,
				    'year_from' => $yearFrom,
				    'year_to' => $yearTo,
				    'year' => $year,
			    ];
		    }

		    $item->set($models);
		    $this->cache->save($item);
	    }

	    return $item->get();
    }

	/**
	 * Vraća model po slug-u.
	 *
	 * @param string $slug
	 * @return string[]
	 */
	public function getModel($slug)
	{
		$cacheKey = sprintf('models.slug.%s', md5($slug));
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$slugParts = explode('--', $slug);

			if (count($slugParts) !== 2) {
				throw new \RuntimeException(sprintf(
					"Neispravan slug modela '%s'.", $slug
				));
			}

			$manufacturer = $this->getManufacturer($slugParts[0]);

			foreach ($this->getModels($manufacturer) as $model) {
				if ($slug === $model['slug']) {
					$item->set($model);
					$this->cache->save($item);
					break;
				}
			}
		}

		return $item->get();
	}

	/**
	 * Vraća model po id-u.
	 *
	 * @param $manufacturerId
	 * @param $modelId
	 * @return string[]
	 */
    public function getModelById($manufacturerId, $modelId)
    {
    	$cacheKey = sprintf('models.id.%d', $modelId);
    	$item = $this->cache->getItem($cacheKey);

    	if (!$item->isHit()) {
		    foreach ($this->getModels($manufacturerId) as $model) {
		    	if ($modelId == $model['id']) {
		    		$item->set($model);
		    		$this->cache->save($item);
		    		break;
			    }
		    }
	    }

	    return $item->get();
    }

    //</editor-fold>
	//<editor-fold desc="Vozila">

    /**
     * Vraća vozila po proizvođaču i modelu.
     *
     * @param int $manufacturerId Id proizvođača.
     * @param int $modelId        Id modela.
     * @return array[]
     */
    public function getVehicles($manufacturerId, $modelId)
    {
    	$cacheKey = sprintf('vehicles.%d.%d', $manufacturerId, $modelId);
    	$item = $this->cache->getItem($cacheKey);

    	if (!$item->isHit()) {
		    $vehicleIds = $this->request('getVehicleIdsByCriteria', [
			    'manuId' => intval($manufacturerId),
			    'modId' => intval($modelId),
		    ]);

		    $vehicleIds = array_map(function($vehicle) {
			    return $vehicle['carId'];
		    }, $vehicleIds);

		    $vehicles = $this->getVehiclesByIds($vehicleIds, $manufacturerId, $modelId);

		    $item->set($vehicles);
	    }

	    return $item->get();
    }

	/**
	 * Vraća detaljna vozila po id-evima.
	 *
	 * @param array $vehicleIds
	 * @param $manufacturer
	 * @param $model
	 *
	 * @return array[]
	 */
	protected function getVehiclesByIds(array $vehicleIds, $manufacturer, $model)
	{
		if (!is_array($manufacturer)) {
			$manufacturer = $this->getManufacturer($manufacturer);
		}

		if (!is_array($model)) {
			$model = $this->getModelById($manufacturer['id'], $model);
		}

		$vehicles = [];
		$chunks = array_chunk($vehicleIds, 25);

		foreach ($chunks as $chunk) {
			$chunk = $this->request('getVehicleByIds4', [
				'axles' => true,
				'cabs' => true,
				'carIds' => ['array' => $chunk],
				'motorCodes' => true,
				'secondaryTypes' => true,
				'wheelbases' => true,
			]);

			$chunk = array_map(function($vehicle) use($manufacturer, $model) {
				$slug = sanitize_title_with_dashes(remove_accents(implode('-', [
					$vehicle['vehicleDetails']['typeName'],
					$vehicle['vehicleDetails']['powerKwFrom'] . 'kw',
				])));
				$slug = sprintf('%s--%s', $model['slug'], $slug);

				$image = isset($vehicle['vehicleDocId']) ? $this->getImageUrl($vehicle['vehicleDocId']) : null;

				return [
					'id' => $vehicle['carId'],
					'manufacturerId' => $manufacturer['id'],
					'modelId' => $model['id'],
					'name' => $vehicle['vehicleDetails']['typeName'],
					'slug' => $slug,
					'constructionType' => $vehicle['vehicleDetails']['constructionType'],
					'engine' => $vehicle['vehicleDetails']['motorType'],
					'power' => $vehicle['vehicleDetails']['powerKwFrom'] . ' kw',
					'powerHp' => $vehicle['vehicleDetails']['powerHpFrom'] . ' ks',
					'image' => $image,
				];
			}, $chunk);

			$vehicles = array_merge($vehicles, $chunk);
		}

		return $vehicles;
	}

	/**
	 * Vraća vozilo po slug-u.
	 *
	 * @param string $slug
	 * @return mixed[]
	 */
	public function getVehicle($slug)
	{
		$cacheKey = sprintf('vehicles.slug.%s', md5($slug));
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$slugParts = explode('--', $slug);

			if (count($slugParts) !== 3) {
				throw new \RuntimeException(sprintf(
					"Neispravan slug vozila '%s'.", $slug
				));
			}

			array_pop($slugParts);
			$manufacturer = $this->getManufacturer($slugParts[0]);
			$model = $this->getModel(implode('--', $slugParts));

			foreach ($this->getVehicles($manufacturer['id'], $model['id']) as $vehicle) {
				if ($slug == $vehicle['slug']) {
					$item->set($vehicle);
					$this->cache->save($item);
					break;
				}
			}
		}

		return $item->get();
	}

	/**
	 * @param $manufacturerId
	 * @param $modelId
	 * @param $vehicleId
	 *
	 * @return mixed
	 */
	public function getVehicleById($manufacturerId, $modelId, $vehicleId)
	{
		$cacheKey = sprintf('vehicles.id.%d', $vehicleId);
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			foreach ($this->getVehicles($manufacturerId, $modelId) as $vehicle) {
				if ($vehicleId == $vehicle['id']) {
					$item->set($vehicle);
					$this->cache->save($item);
					break;
				}
			}
		}

		return $item->get();
	}

	public function getVehicleIdsLinkedToArticle($articleId)
	{
		$cacheKey = sprintf('vehicles.article_id.vehicles.%d', $articleId);
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$vehicleIds = [];
			$articleLinks = $this->request('getArticleLinkedAllLinkingTarget4', [
				'articleId' => $articleId,
			]);

			if (isset($articleLinks[0]['articleLinkages']['array'])) {
				$linkedArticlePairs = array_map(function($link) {
					return [
						'articleLinkId' => $link['articleLinkId'],
						'linkingTargetId' => $link['linkingTargetId'],
					];
				}, $articleLinks[0]['articleLinkages']['array']);

				$chunks = array_chunk($linkedArticlePairs, 25);

				foreach ($chunks as $chunk) {
					$vehicles = $this->request('getArticleLinkedAllLinkingTargetsByIds3', [
						'articleId' => $articleId,
						'immediateAttributs' => true,
						'linkedArticlePairs' => ['array' => $chunk],
					]);

					$vehiclesArray = array_map(function($vehicle) {
						return $vehicle['linkedVehicles']['array'];
					}, $vehicles);

					foreach ($vehiclesArray as $_vehicles) {
						foreach ($_vehicles as $vehicle) {
							$vehicleIds[] = $vehicle['carId'];
						}
					}
				}
			}

			$item->set($vehicleIds);
			$this->cache->save($item);
		}

		return $item->get();
	}

	public function getVehicleStringsLinkedToArticle($articleId)
	{
		$cacheKey = sprintf('vehicles.article_id.%d', $articleId);
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$allVehicles = [];
			$articleLinks = $this->request('getArticleLinkedAllLinkingTarget4', [
				'articleId' => $articleId,
			]);

			if (isset($articleLinks[0]['articleLinkages']['array'])) {
				$linkedArticlePairs = array_map(function($link) {
					return [
						'articleLinkId' => $link['articleLinkId'],
						'linkingTargetId' => $link['linkingTargetId'],
					];
				}, $articleLinks[0]['articleLinkages']['array']);

				$chunks = array_chunk($linkedArticlePairs, 25);

				foreach ($chunks as $chunk) {
					$vehicles = $this->request('getArticleLinkedAllLinkingTargetsByIds3', [
						'articleId' => $articleId,
						'immediateAttributs' => true,
						'linkedArticlePairs' => ['array' => $chunk],
					]);

					$vehiclesArray = array_map(function($vehicle) {
						return $vehicle['linkedVehicles']['array'];
					}, $vehicles);

					foreach ($vehiclesArray as $_vehicles) {
						foreach ($_vehicles as $vehicle) {
							if (!isset($allVehicles[$vehicle['manuDesc']])) {
								$allVehicles[$vehicle['manuDesc']] = [];
							}

							$allVehicles[$vehicle['manuDesc']][] = sprintf(
								'%s %s %s %s %d kw',
								$vehicle['manuDesc'],
								$vehicle['modelDesc'],
								$vehicle['constructionType'],
								$vehicle['carDesc'],
								$vehicle['powerKwFrom']
							);
						}
					}
				}
			}

			$item->set($allVehicles);
			$this->cache->save($item);
		}

		return $item->get();
	}

	//</editor-fold>
	//<editor-fold desc="Artikli">

	public function getArticlesByLinks(array $articleLinks)
	{
		if (!($vehicle = $this->getSelectedVehicle())) {
			return [];
		}

		$cacheKey = sprintf('articles.links.%s', md5(serialize($articleLinks)));
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$articles = [];
			$chunks = array_chunk($articleLinks, 25);

			foreach ($chunks as $chunk) {
				$chunk = $this->request('getAssignedArticlesByIds7', [
					'articleIdPairs' => ['array' => $chunk],
					'linkingTargetId' => $vehicle['id'],
					'manuId' => $vehicle['manufacturerId'],
					'modId' => $vehicle['modelId'],
					'attributs' => true,
					'basicData' => true,
					'documents' => true,
					'eanNumbers' => true,
					'immediateAttributs' => true,
					'immediateInfo' => true,
					'info' => true,
					'mainArticles' => true,
					'oeNumbers' => true,
					'replacedByNumbers' => true,
					'replacedNumbers' => true,
					'thumbnails' => true,
					'usageNumbers' => true,
				]);

				$articles = array_merge($articles, $chunk);
			}

			$articles = array_map([$this, 'mapArticle'], $articles);

			$item->set($articles);
			$this->cache->save($item);
		}

		return $item->get();
	}

	public function getArticlesByIds(array $articleIds)
	{
		sort($articleIds);
		$cacheKey = sprintf('articles.ids.%s', md5(implode(',', $articleIds)));
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$articles = [];
			$chunks = array_chunk($articleIds, 25);

			foreach ($chunks as $chunk) {
				$chunk = $this->request('getDirectArticlesByIds4', [
					'articleId' => ['array' => $chunk],
					'attributs' => true,
					'basicData' => true,
					'documents' => true,
					'eanNumbers' => true,
					'immediateAttributs' => true,
					'immediateInfo' => true,
					'info' => true,
					'mainArticles' => true,
					'normalAustauschPrice' => false,
					'oeNumbers' => true,
					'prices' => false,
					'replacedByNumbers' => true,
					'replacedNumbers' => true,
					'thumbnails' => true,
					'usageNumbers' => true,
				]);

				$articles = array_merge($articles, $chunk);
			}

			$articles = array_map([$this, 'mapArticle'], $articles);

			$item->set($articles);
			$this->cache->save($item);
		}

		return $item->get();
	}

	public function mapArticle($article)
	{
		if (!isset($article['directArticle']) && isset($article['assignedArticle'])) {
			$article['directArticle'] = $article['assignedArticle'];
		}

		$images = isset($article['articleDocuments']['array'])
			? array_filter(array_map(function($document) {
				return in_array(intval($document['docTypeId']), [1, 5])
					? $this->getImageUrl($document['docId'], $document['docFileName'])
					: false;
			}, $article['articleDocuments']['array']))
			: [];

		$fittingPositions = [];

		$attributes = isset($article['articleAttributes']['array'])
			? array_map(function($attribute) use (&$fittingPositions) {
				$value = isset($attribute['attrValue']) ? $attribute['attrValue'] : '';
				$unit = isset($attribute['attrUnit']) ? $attribute['attrUnit'] : '';

				$mapped = [
					'id' => $attribute['attrId'],
					'name' => $attribute['attrName'],
					'value' => trim(sprintf('%s %s', $value, $unit)),
				];

				if (100 === $attribute['attrId']) {
					$fittingPositions[] = $mapped;
				}

				return $mapped;
			}, $article['articleAttributes']['array'])
			: [];

		$attributes = array_merge($fittingPositions, $attributes);

		$infos = isset($article['articleInfo']['array'])
			? array_map(function($info) {
				return [
					'id' => $info['infoId'],
					'type' => $info['infoType'],
					'name' => $info['infoTypeName'],
					'text' => preg_replace('#(https?://[^ ]+)#', '<a href="\\1" target="_blank">\\1</a>', $info['infoText']),
				];
			}, $article['articleInfo']['array'])
			: [];

		$oenumbers = isset($article['oenNumbers']['array'])
			? array_map(function($number) {
				return [
					'brand' => $number['brandName'],
					'number' => $number['oeNumber'],
				];
			}, $article['oenNumbers']['array'])
			: [];

		return [
			'id' => $article['directArticle']['articleId'],
			'genericArticleId' => $article['directArticle']['genericArticleId'],
			'key' => sprintf('%d|%s', $article['directArticle']['brandNo'], $article['directArticle']['articleNo']),
			'name' => $article['directArticle']['articleName'],
			'attributes' => $attributes,
			'infos' => $infos,
			'images' => $images,
			'brand' => [
				'id' => $article['directArticle']['brandNo'],
				'name' => $article['directArticle']['brandName'],
			],
			'oenumbers' => $oenumbers,
		];
	}

	/**
	 * Vraća id artikla po ključu.
	 *
	 * @param $articleKey
	 * @return null
	 */
	public function getArticleId($articleKey)
	{
		if (!$articleKey || preg_match('/^\d*\|$/', $articleKey)) {
			return null;
		}

		$cacheKey = sprintf('articles.search.key.%s', md5($articleKey));
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$articleKeyArray = explode('|', $articleKey);
			if (count($articleKeyArray) !== 2) return null;

			list($brandId, $articleNumber) = explode('|', $articleKey);

			if (!$articleNumber) {
				var_dump($articleKey);
			}

			$articles = $this->request('getArticleDirectSearchAllNumbersWithState', [
				'brandId' => $brandId,
				'articleNumber' => $articleNumber,
				'numberType' => 10,
				'searchExact' => true,
			]);

			foreach ($articles as $article) {
				if ($article['articleNo'] == $articleNumber && $article['brandNo'] == $brandId) {
					$article = [
						'id' => $article['articleId'],
					];
					$item->set($article);
					$this->cache->save($item);
					break;
				}
			}
		}

		$article = $item->get();
		return $article ? $article['id'] : null;
	}

	/**
	 * Vraća artikle po generic article id-evima.
	 *
	 * @param int[] $genericArticleIds
	 * @return array
	 */
	public function getArticlesByGenericIds($genericArticleIds)
	{
		if (!($vehicle = $this->getSelectedVehicle())) {
			return [];
		}

		if (!is_array($genericArticleIds)) {
			$genericArticleIds = [$genericArticleIds];
		}

		sort($genericArticleIds);

		$cacheKey = sprintf('articles.vehicle.%d.generic.%s', $vehicle['id'], md5(implode(';', $genericArticleIds)));
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$articleLinks = [];

			$chunks = array_chunk($genericArticleIds, 25);

			foreach ($chunks as $chunk) {
				$data = $this->request('getArticleIdsWithState', [
					'linkingTargetId' => $vehicle['id'],
					'genericArticleId' => ['array' => $chunk],
				]);

				$articleLinks = array_merge($articleLinks, array_map(function($article) {
					return [
						'articleId' => $article['articleId'],
						'articleLinkId' => $article['articleLinkId'],
					];
				}, $data));
			}

			$articles = $this->getArticlesByLinks($articleLinks);

			$item->set($articles);
			$this->cache->save($item);
		}

		return $item->get();
	}

	/**
	 * Vrać artikle po assembly grupu.
	 *
	 * @param $assemblyGroupIds
	 *
	 * @return array|mixed
	 */
	public function getArticlesByAssemblyGroupIds($assemblyGroupIds)
	{
		if (!($vehicle = $this->getSelectedVehicle())) {
			return [];
		}

		if (!is_array($assemblyGroupIds)) {
			$assemblyGroupIds = [$assemblyGroupIds];
		}

		sort($assemblyGroupIds);

		$cacheKey = sprintf('articles.vehicle.%d.%s', $vehicle['id'], md5(implode(';', $assemblyGroupIds)));
		$item = $this->cache->getItem($cacheKey);

		if (!$item->isHit()) {
			$articleIds = [];

			foreach ($assemblyGroupIds as $assemblyGroupId) {
				$articleIdsForGroup = $this->request('getArticleIdsWithState', [
					'linkingTargetId' => $vehicle['id'],
					'assemblyGroupNodeId' => $assemblyGroupId,
				]);

				$articleIdsForGroup = array_map(function($article) {
					return $article['articleId'];
				}, $articleIdsForGroup);

				$articleIds = array_merge($articleIds, $articleIdsForGroup);
			}

			$articleIds = array_unique($articleIds);
			$articles = $this->getArticlesByIds($articleIds);

			$item->set($articles);
			$this->cache->save($item);
		}

		return $item->get();
	}

	public function searchArticlesByBrandAndNumber($brandId, $articleNumber)
	{

	}

	/**
	 * Vraća ključeve artikala po broju.
	 *
	 * @param string $articleNumber Broj artikla.
	 *
	 * @return string[]
	 */
	public function searchArticlesByNumber($articleNumber)
	{
		$articles = $this->request('getArticleDirectSearchAllNumbersWithState', [
			'articleNumber' => $articleNumber,
			'searchExact' => true,
			'numberType' => 10,
		]);

		return array_map(function($article) {
			return sprintf('%d|%s', $article['brandNo'], $article['articleNo']);
		}, $articles);
	}

	//</editor-fold>

	/**
	 * @param $documentId
	 * @param null $fileName
	 * @return string
	 */
	public function getImageUrl($documentId, $fileName = null)
	{
		return sprintf(
			'%s/pegasus-3-0/documents/%s/%s',
			preg_replace('#^[^/]+#', '', self::SERVICE_URI),
			$this->config['provider'],
			$documentId /*. ($fileName ? '/' . $fileName : '')*/
		);
	}

	/**
	 * Formatira godinu proizvodnje.
	 *
	 * @param string $year
	 * @param string $default
	 *
	 * @return string
	 */
    protected function formatConstructionYear($year, $default = '...')
    {
	    if (preg_match('/(\d{4})(\d{2})/', $year, $matches)) {
		    return sprintf('%s/%s', $matches[2], $matches[1]);
	    }

	    return $default;
    }
}